use std::mem::{align_of, size_of};

use zerocopy::{FromBytes, FromZeroes, Ref};

use crate::{FilePointer, FileRange, RawFilePointer};

pub trait ReaderTrait {
    fn read_raw(&self, range: FileRange) -> &[u8];
    fn read<T: FromBytes>(&self, ptr: FilePointer<T>) -> &T {
        debug_assert!(!ptr.is_null(), "null pointer dereference");
        let bytes = self.read_raw(ptr.range());
        Ref::<_, T>::new(bytes)
            .unwrap_or_else(|| {
                panic!(
                    "failed to interpret {:?} = {bytes:?} as {}",
                    ptr.range(),
                    std::any::type_name::<T>()
                )
            })
            .into_ref()
    }
}
